/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/////////////////////////////////////////////////////////////////////////////////
//
//
//   File : HashTable.cpp
//
//   Created : 09/22/98
//   By : Patrick McDaniel
//
//   Description : (See HashTable.h for class description)
//
//   *** Change History ***
//
//   09/24/98 Created File : DPM
//   10/18/98 Added CRL m/cast support : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>
#include <stdlib.h>

// Project Include Files
#include <dbgLog.h>
#include <HashTable.h>
#include <List.h>
#include <LinkedList.h>

// Definitions

// 
// Member Functions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::HashTable
// Description  : Default Constructor
//
// Inputs       : htype - type of hash used
//                tablesize - size of hash table
// Outputs      : none

HashTable::HashTable( int htype, int tablesize )
  :  List( htype, 0 ),
     hTable( NULL ),
     tableSize( tablesize ),
     iteratorIndex( 0 ),
     listSize( 0 )
{
    // Create and zero the table
    if ( tableSize > 0 ) {
	hTable = new List *[tableSize];
        memset( hTable, 0x0, sizeof(List *)*tableSize );
    }
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::~HashTable
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

HashTable::~HashTable( void )
{
    // Destroy all objects in the list, then the list
    for ( int i = 0; i<tableSize; i++ )
      delete hTable[i];
    delete hTable;

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::addElement
// Description  : Add an element to the list
//
// Inputs       : ele - element to add
// Outputs      : 0 if successful, -1 if failure

int HashTable::addElement( Listable *ele )
{
    // Get the index number, then add it
    int idx = ele->hashValue() % tableSize;
    if ( hTable[idx] == NULL ) 
        hTable[idx] = new LinkedList( HashType() );
    hTable[idx]->addElement( ele );

    // Increment list size
    listSize ++;

    // Return succesfully
    return( 0 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::removeElement
// Description  : Remove an element from the list
//
// Inputs       : ele - element to remove
// Outputs      : removed element, or NULL if not found

Listable *HashTable::removeElement( Listable *ele )
{
    // If bad element
    if ( ele == NULL ) return( NULL );

    // Get the index number, attempt to remove item
    int idx = ele->hashValue() % tableSize;
    Listable *lst = (hTable[idx] == NULL) ? 
                    (Listable *)NULL : hTable[idx]->removeElement(ele);

    // Decrement list size
    if ( lst != NULL ) listSize --;

    // Return (not found)
    return( lst );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::removeElement
// Description  : Remove an element from the list (by value)
//
// Inputs       : ele - element to remove
// Outputs      : removed element, or NULL if not found

Listable *HashTable::removeElement( void *value )
{
    // Get the index number, attempt to find item
    int idx = hashValue(value) % tableSize;
    Listable *lst = (hTable[idx] == NULL) ? 
                    (Listable *)NULL : hTable[idx]->removeElement(value);

    // Return removed (if any) element
    return( lst );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::findElement
// Description  : Find and return an element from list
//
// Inputs       : ele - element to find
// Outputs      : element or NULL if not found

Listable *HashTable::findElement( Listable *ele )
{
    // Get the index number, attempt to find item
    int idx = ele->hashValue() % tableSize;
    Listable *lst = (hTable[idx] == NULL) ? 
                    (Listable *)NULL : hTable[idx]->findElement(ele);

    // Return, whatever (if any) element found
    return( lst );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::findElement
// Description  : Find and return an element from list
//
// Inputs       : value - value of element to find
// Outputs      : element or NULL if not found

Listable *HashTable::findElement( void *value )
{
    // Get the index number, attempt to find item
    int idx = hashValue(value) % tableSize;
    Listable *lst = (hTable[idx] == NULL) ? 
                    (Listable *)NULL : hTable[idx]->findElement(value);

    // Return, whatever (if any) element found
    return( lst );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::numElements
// Description  : Number of elements in the list
//
// Inputs       : none
// Outputs      : number of elements in list

int HashTable::numElements( void )
{
    return( listSize );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::resetIterator
// Description  : Reset the iterator to beginning of list.
//
// Inputs       : none
// Outputs      : none

void HashTable::resetIterator( void )
{
    // Reset iterator and return
    iteratorIndex = 0;
    if ( hTable[0] != NULL ) hTable[0]->resetIterator();
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::iterator
// Description  : Interate function over list
//
// Inputs       : none
// Outputs      : next element in list

Listable *HashTable::iterator( void )
{

    // If already at end of list, return NULL
    if ( iteratorIndex >= tableSize )
        return( NULL );

    // Loop until you get to the end of the table
    while ( iteratorIndex < tableSize ) 
    {
        // Check the current table entry
        Listable *ptr;
        if ( (hTable[iteratorIndex] != NULL) &&
             ((ptr = hTable[iteratorIndex]->iterator()) != NULL) )
            return( ptr );
        
        if ( (++iteratorIndex < tableSize) && (hTable[iteratorIndex] != NULL) )
            hTable[iteratorIndex]->resetIterator();
    }

    // Reached end of list, return NULL
    return( NULL );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : HashTable::printList
// Description  : print to the debug log the contents of the linked list
//
// Inputs       : lvl - debug level to print
//                txt - text to print describing the list
// Outputs      : none

void HashTable::printList( int lvl, const unsigned char *txt )
{
    // If debug level not on, don't bother
    if ( !LOG_LEVEL_ON(lvl) ) return;

    // Print some header
    _dbgLogMsg( lvl, "Begin Hash Table (%d) [%s]", listSize, txt );


    // Print each table entry
    int start = -1, i;
    for ( i=0; i<tableSize; i++ )
    {
        if ( (hTable[i] == NULL) || (hTable[i]->numElements() == 0) )
        {
            if ( start == -1 ) start = i;
        }
        else
        {
            // Print the empty indices
            if ( start != -1 )
            {
                _dbgLogMsg( lvl, "Table indices [%d-%d] (empty)", start, i-1 );
                start = -1;
            }

            // Print the list
            unsigned char tempstr[2048], *ptr;
            Listable *lst;
            int first = 1;
            sprintf( (char *)tempstr, "Table index [%d] - ", i );
            hTable[i]->resetIterator();
            while ( (lst = hTable[i]->iterator()) != NULL )
            {
                ptr = lst->toString();
                if ( strlen((char *)tempstr) + 2 + strlen((char *)ptr) < 2047 )
                {
                    if ( !first ) 
                        strcat( (char *)tempstr, ", " );
                    else
                        first = 0;
                    strcat( (char *)tempstr, (char *)ptr );
                }
            }
            _dbgLogMsg( lvl, (char *)tempstr );
        }

    }

    // Print the empty indices
    if ( start != -1 )
        _dbgLogMsg( lvl, "Table indices [%d-%d] (empty)", start, i-1 );

    // Print some trailer
    _dbgLogMsg( lvl, "End Hash Table [%s]", txt );

    // Return, no return code
    return;
}
