/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : LinkedList.cpp
//
//   Created : 09/22/98
//   By : Patrick McDaniel
//
//   Description : (See LinkedList.h for class description)
//
//   *** Change History ***
//
//   09/22/98 Created File : DPM
//   10/13/98 Delete structus and Listables in destructor : DPM
//   11/03/98 Finished pre-release : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdlib.h>

// Project Include Files
#include <dbgLog.h>
#include <LinkedList.h>

// Definitions

// 
// Member Functions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::LinkedList
// Description  : Default Constructor
//
// Inputs       : none
// Outputs      : none

LinkedList::LinkedList( int htype, int sorted )
  :  List( htype, sorted ),
     head( NULL ),
     tail( NULL ),
     interatorPtr( NULL ),
     listSize( 0 )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::~LinkedList
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

LinkedList::~LinkedList( void )
{
    // Destroy all objects in the list
    listable *ptr = head, *trail;
    while ( ptr != NULL )
    {
        // Walk through the list, freeing structs and Listables
        trail = ptr;
        ptr = ptr->next;
        delete trail->value;
        delete trail;
    }

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::addElement
// Description  : Add an element to the list
//
// Inputs       : ele - element to add
// Outputs      : 0 if successful, -1 if failure

int LinkedList::addElement( Listable *ele )
{
    // Create new buffer
    listable *lst = new listable;
    lst->next = NULL;
    lst->value = ele;

    // If sorted
    if ( isSorted )
    {
        // If list empty
        if ( head == NULL )
        {
            // Set pointers, increment list size, and return
            head = tail = lst;
            listSize ++;
            return( 0 );
        }

        // Find the correct placement in the list
        listable *ptr = head, *trail = NULL;
        while ( (ptr != NULL) && 
                (ptr->value->compareListValues(lst->value->listValue()) < 0) )
        {
            trail = ptr;
            ptr = ptr->next;
        }

        // If trail == NULL, replace head
        if ( trail == NULL )
        {
            lst->next = head;
            head = lst;
        } else

        // If at end of list
        if ( ptr == NULL )
        {
            tail->next = lst;
            tail = lst;
        } else
        {
            // Somewhere in the middle
            lst->next = trail->next;
            trail->next = lst;
        }
    }
    else // Unsorted
    {
        // If tail == NULL, then list empty
        if ( tail == NULL )
            head = lst;
        else
            tail->next = lst;
        tail = lst;
    }

    // Increment list size
    listSize ++;

    // Return succesfully
    return( 0 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::removeElement
// Description  : Remove an element from the list
//
// Inputs       : ele - element to remove
// Outputs      : removed element, or NULL if not found

Listable *LinkedList::removeElement( Listable *ele )
{
    // If list empty (or NULL element)
    if ( (head == NULL) || (ele == NULL) ) return( NULL );

    // Find the correct element
    listable *ptr = head, *trail = NULL;
    while ( (ptr != NULL) && (ptr->value != ele) )
    {
        trail = ptr;
        ptr = ptr->next;
    }

    // If ptr == NULL, not found
    if ( ptr == NULL ) return( NULL );

    // If trail == NULL, remove head (implcitly, ptr == head)
    if ( trail == NULL )
    {
        if ( head == tail ) tail = NULL; // Only one element
        head = head->next;
    }
    else
        trail->next = ptr->next;

    // If ptr == tail (removing tail)
    if ( ptr == tail ) tail = trail;

    // Decrement list size
    listSize --;

    // Return (found)
    Listable *ltb = ptr->value;
    delete ptr;
    return( ltb );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::removeElement
// Description  : Remove an element from the list (by value)
//
// Inputs       : ele - element to remove
// Outputs      : removed element, or NULL if not found

Listable *LinkedList::removeElement( void *value )
{
    // Find, then remove it
    return( removeElement( findElement(value) ) );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::findElement
// Description  : Find and return an element from list
//
// Inputs       : ele - element to find
// Outputs      : element or NULL if not found

Listable *LinkedList::findElement( Listable *ele )
{
    // Search the list, return element
    listable *ptr = head;
    while ( (ptr != NULL) && (ptr->value != ele) )
        ptr = ptr->next;

    // Return, whatever (if any) element found
    return( (ptr == NULL) ? (Listable *)NULL : ptr->value );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::findElement
// Description  : Find and return an element from list
//
// Inputs       : value - value of element to find
// Outputs      : element or NULL if not found

Listable *LinkedList::findElement( void *value )
{
    // Search based on sorted/not sorted
    int retval;
    listable *ptr = head;

    // Sorted search
    if ( isSorted )
    {
       // Search the list (only as far as we need to), return element
        while ( (ptr != NULL) && 
                ((retval = ptr->value->compareListValues(value)) < 0) )
            ptr = ptr->next;
        return( (retval == 0) ? ptr->value : (Listable *)NULL );
    }

    // Search the list, return element
    while ( (ptr != NULL) && 
            (ptr->value->compareListValues(value) != 0) )
        ptr = ptr->next;

    // Return, whatever (if any) element found
    return( ptr->value );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::numElements
// Description  : Number of elements in the list
//
// Inputs       : none
// Outputs      : number of elements in list

int LinkedList::numElements( void )
{
    return( listSize );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::resetIterator
// Description  : Reset the iterator to beginning of list.
//
// Inputs       : none
// Outputs      : none

void LinkedList::resetIterator( void )
{
    // Reset iterator and return
    interatorPtr = head;
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::iterator
// Description  : Interate function over list
//
// Inputs       : none
// Outputs      : next element in list

Listable *LinkedList::iterator( void )
{
    // If already at end of list, return NULL
    if ( interatorPtr == NULL )
        return( NULL );

    // Not at end of list, return next and advance iterator
    Listable *ptr = interatorPtr->value;
    interatorPtr = interatorPtr->next;
    return( ptr );
}

//
// Debug Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : LinkedList::printList
// Description  : print to the debug log the contents of the linked list
//
// Inputs       : lvl - debug level to print
//                txt - text to print describing the list
// Outputs      : none

void LinkedList::printList( int lvl, const unsigned char *txt )
{
    // If debug level not on, don't bother
    if ( !LOG_LEVEL_ON(lvl) ) return;

    // Print some header
    _dbgLogMsg( lvl, "Begin Linked List (%d) [%s]", listSize, txt );

    // If empty list
    if ( head == NULL )
        _dbgLogMsg( lvl, "  <empty>" );
    else
    {
        // Traverse the list, printing values
        listable *ptr = head;
        while ( ptr != NULL )
        {
            _dbgLogMsg( lvl, "  %s", ptr->value->toString() );
            ptr = ptr->next;
        }
    }

    // Print some trailer
    _dbgLogMsg( lvl, "End Linked List [%s]", txt );

    // Return, no return code
    return;
}

//
// Member Functions

