/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : List.cpp
//
//   Created : 10/06/98
//   By : Patrick McDaniel
//
//   Description : (See List.h for class description)
//
//   *** Change History ***
//
//   10/06/98 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>
#include <string.h>

// Project Include Files
#include <dbgLog.h>
#include <List.h>
#include <Listable.h>
#include <Support.h>

// Definitions

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : List::atIndex
// Description  : This is a generic indexing function.  It is very
//                ineffecient and should be overridden. 
//                (0 based indexing)
//
// Inputs       : idx - index into list
// Outputs      : Listable * if successful, NULL if failure

Listable * List::atIndex( int idx )
{
    // Local variables
    int index = -1;
    Listable *lst = NULL;

    // If out of range return NULL
    if ( (idx < 0) || (idx >= numElements()) )
        return( NULL );

    // Use the iterator, travere idx into list
    resetIterator();
    while ( index != idx )
    {
        // If reached end of list, return NULL
        if ( (lst = iterator()) == NULL )
            return( NULL );
        index ++;
    }

    // Return the found item
    return( lst );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : List::ListString
// Description  : Convert list into string
//
// Inputs       : none
// Outputs      : pointer to (static) buffer containing new string

char * List::ListString( void )
{
    static char buf[1024];

    // If empty CRL, print it
    if ( numElements() == 0 )
        sprintf( buf, "<Empty>" );
    else
    {
        // Print as much of the list as we can
        Listable *lst;
        buf[0] = 0x0;
        resetIterator();
        while ( (lst = iterator()) != NULL )
        {
            // If not enough room
            char *str = (char *)lst->toString();
            if ( strlen(buf) + strlen(str) + 2 >= 1024 )
            {
                // Add an idicator of more entries
                if ( strlen(buf) + 3 < 1024 ) 
                    strcat( buf, "..." );
                else 
                    buf[strlen(buf)-1] = '+';
                return( buf );
            }

            // Concatinate the strings
            if ( buf[0] != 0x0 ) strcat( buf, ", " );
            strcat( buf, str );
        }
    }

    // Return the string
    return( buf );
}

//
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : List::HashValue
// Description  : Convert string into hash value
//
// Inputs       : value - pointer to generic values
// Outputs      : hash value

unsigned int List::hashValue( void *value )
{
    // Based on hash type, convert to hash value
    switch ( hashType )
    {
        case STRING_HASH: // String type
             return( Listable::hashValue((char *)value) );

        case UL_HASH: // unsigned long type
             return( *(unsigned long *)value );

        default: // Unknown, debug and error out
             _dbgLogMsg( DBG_FATAL_ERROR, "Unknown list hash type %d, exiting", 
                         hashType );
             SUPASSERT( 0, "Bad hash type" );
             break;
    }

    // Return error (UNREACHABLE)
    return( 0 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : List::addList
// Description  : Add all elements from one list to another.
//
// Inputs       : list - list to add
// Outputs      : this list

List *List::addList( List *list )
{
    // Walk list to add
    list->resetIterator();
    Listable *lst;
    while ( (lst = list->iterator()) != NULL )
        addElement( lst );

    // Return, no return code
    return( this );
}


//////////////////////////////////////////////////////////////////////////////////
//
// Function     : List::cleanList
// Description  : Empty the contents of the list.
//
// Inputs       : none
// Outputs      : none

void List::cleanList( void )
{
    // Remove everything in list
    Listable *lst;
    while ( (lst = atIndex(0)) != NULL )
        removeElement( lst );

    // Return, no return code.
    return;
}
