/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : Metric.cpp
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : (See Metric.h for class description)
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <dbgLog.h>
#include <Support.h>
#include <LinkedList.h>
#include <Metric.h>
#include <CountMetric.h>
#include <MaxMetric.h>
#include <AverageMetric.h>

// Definitions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::Metric
// Description  : Attribute constructor
//
// Inputs       : indx - unique ID for metric
// Outputs      : none

Metric::Metric( int indx )
  : filters( new LinkedList(UL_HASH) ),
    metricId( indx )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::~Metric
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

Metric::~Metric( void )
{
    // Cleanup list, then delete
    filters->cleanList();
    delete filters;

    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::hashValue
// Description  : Hash value of object.
//
// Inputs       : none
// Outputs      : hash value

unsigned int Metric::hashValue( void )
{
    return( metricId );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::listValue
// Description  : Get the ordering list value (not to be interpreted)
//
// Inputs       : none
// Outputs      : pointer to value 

void * Metric::listValue( void )
{
    return( (void *)&metricId );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::compareListValues
// Description  : Make a comparison between the two values.
//
// Inputs       : value - value to compare it to
// Outputs      : < 0 if local value is less than value parameter
//                0 if local value is equal to value parameter
//                > 0 if local value is greater than value parameter

int Metric::compareListValues( void *value )
{
    return( metricId - *((int *)value) );
}

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::unregisterFilter
// Description  : Unregister the filter that was contacted during mark phase.
//
// Inputs       : fil - filter to unregister
// Outputs      : none

void Metric::unregisterFilter( MetricFilter *fil )
{
    // Remove from the list
    filters->removeElement( fil );

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::registerFilter
// Description  : Register the filter to be contacted during mark phase.
//
// Inputs       : fil - filter to register
// Outputs      : none

void Metric::registerFilter( MetricFilter *fil )
{
    // Put it in the list
    filters->addElement( fil );

    // Return, no return code
    return;
}

//
// Static Member Functions

      // Create a matric from the factory.
//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Metric::registerFilter
// Description  : Register the filter to be contacted during mark phase.
//
// Inputs       : id - metric identifier
//                typ - metric type identifier
// Outputs      : none

Metric * Metric::MetricFactory( int id, int typ )
{
    // Switch on metric type
    switch (typ)
    {
        case COUNT_METRIC_TYPE: // Count Metric
             return( new CountMetric( id ) );
             break;

        case MAX_METRIC_TYPE: // Maximum value metric
             return( new MaxMetric( id ) );
             break;

        case AVG_METRIC_TYPE: // Average value metric
             return( new AverageMetric( id ) );
             break;

        default: 
             SUPASSERT( 0, "Unknown metric type." );
    }

    // Unreachable return
    return( NULL );
}
