/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : Utils.cpp
//
//   Created : 09/22/98
//   By : Patrick McDaniel
//
//   Description : (See Utils.h for class description)
//
//   *** Change History ***
//
//   09/22/98 Created File : DPM
//   09/30/98 Added parameter functions : DPM
//   11/03/98 Finished pre-release : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <ctype.h>
#include <stdio.h>
#include <string.h>

// Project Include Files
#include <dbgLog.h>
#include <Utils.h>

// Definitions

// 
// Static Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::getToken
// Description  : get a token from the input line, placing the token
//                in the input buffer.
//
// Inputs       : line - inbuf string to search
//                buf - buffer to place token
//                length - of buffer
// Outputs      : a pointer to the char after the token, or NULL if no token

unsigned char *Utils::getToken( unsigned char *line, unsigned char *buf, 
                                int buflen )
{
    // Local vars
    unsigned char *start;
    int indx = 0;

    // Search through the line, looking for token
    while ( (line[indx] != 0x0) && (isspace(line[indx])) ) indx ++;
    if ( line[indx] == 0x0 )
    {
        // Zero buffer and return NULL
        buf[0] = 0x0;
        return( NULL );
    }
    start = &line[indx]; 
    indx = 1;
    while ( (start[indx] != 0x0) && (!isspace(start[indx])) ) indx++;

    // Copy buffer, null terminate
    int cpbytes = (indx<buflen) ? indx : buflen;
    strncpy( (char *)buf, (char *)start, cpbytes );
    buf[cpbytes] = 0x0;

    // Return, no return code
    return( &start[indx] );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::tolower
// Description  : convert a string to lower case
//
// Inputs       : line - buffer to convert
// Outputs      : a pointer to the char after the conversion

unsigned char *Utils::toLower( unsigned char *line )
{
    // Convert each char
    int slen = strlen((char *)line);
    for ( int i=0; i<slen; i++ ) line[i] = tolower(line[i]);

    // Return sucessfully
    return( line );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::hashValue
// Description  : compute a hash value for a string
//
// Inputs       : str - string to convert
// Outputs      : the hash value of the string

int Utils::hashValue( unsigned char *str )
{
    // Convert each char
    int hValue = 0;
    for ( int i=0; i<(int)strlen((char *)str); i++ ) 
        hValue += (int)str[i];

    // Return sucessfully
    return( hValue );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (has no value, just 
//                check existance)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : -1 - if comment line
//                0 - if parameter not found
//                1 - if parameter found 
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s", name );

    // Return, value read
    return( 1 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (depositing the 
//                value in the value reference)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : -1 - if comment line
//                0 - if parameter no found
//                1 - if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, int *value )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Get parse number token (or throw exception on bad value)
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%d", value ) != 1)
        throw new DataFormatException( name );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s [%d]", name, *value );

    // Return, value read
    return( 1 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (depositing the 
//                value in the value reference)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : -1 - if comment line
//                0 - if parameter no found
//                1 - if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, unsigned long *value )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Get parse number token (or throw exception on bad value)
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%ld", value ) != 1)
        throw new DataFormatException( name );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s [%ld]", name, *value );

    // Return, value read
    return( 1 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (depositing the 
//                value in the value reference)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : -1 - if comment line
//                0 - if parameter no found
//                1 - if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, double *value )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Get parse number token (or throw exception on bad value)
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%lf", value ) != 1)
        throw new DataFormatException( name );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s [%lf]", name, *value );

    // Return, value read
    return( 1 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (depositing the 
//                value in the value reference)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : 0 - if parameter no found
//                1 - if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, unsigned char **value )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Get parse string token (or throw exception on bad value)
    if ( Utils::getToken( lnPtr, tok, 256 ) == NULL )
        throw new DataFormatException( name );
    *value = (unsigned char *)strdup( (char *)tok );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s [%s]", name, *value );

    // Return, value read
    return( 1 );
}



//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read multiparameter on line
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                val1, val2 - reference to put values 
// Outputs      : -1 - if comment line
//                0 - if parameter no found
//                1 - if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, 
                          double *val1, double *val2 )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Get parse number token (or throw exception on bad value)
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%lf", val1 ) != 1)
        throw new DataFormatException( name );
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%lf", val2 ) != 1)
        throw new DataFormatException( name );

    // Return successfully
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading %s [%lf,%lf]", name, *val1, *val2 );

    // Return, value read
    return( 1 );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Utils::readParameter
// Description  : Read the parameter from the input line (depositing the 
//                value in the value reference)
//
// Inputs       : line - input to read parameter from
//                name - name of parameter
//                value - reference to put value in
// Outputs      : -1 - if comment line
//                0 - if parameter no found
//                length of array if parameter found and read
//                throws exception if bad input line

int Utils::readParameter( unsigned char *line, char *name, unsigned long **value )
{
    // Get the first token, ignore blank or comment lines
    unsigned char tok[256], *lnPtr;
    lnPtr = Utils::getToken( line, tok, 256 );

    // Check for comment line
    if ( (lnPtr == NULL ) || (tok[0] == '#') )
        return( -1 );

    // If No match
    if ( strcmp( (char *)toLower(tok), name ) != 0 )
        return( 0 );

    // Read the array length
    int arLen;
    lnPtr = Utils::getToken( lnPtr, tok, 256 );
    if ( sscanf( (char *)tok, "%d", &arLen ) != 1)
        throw new DataFormatException( name );
    _dbgLogMsg( DBG_READ_DEFINITION, "  Reading array length %s %d", name, arLen );

    // Create the array
    *value = new unsigned long[arLen];

    // Read the array contents
    int cnt = 0;
    while ( cnt < arLen )
    {
        // Get parse number token (or throw exception on bad value)
        lnPtr = Utils::getToken( lnPtr, tok, 256 );
        if ( sscanf( (char *)tok, "%ld", &(*value)[cnt] ) != 1)
            throw new DataFormatException( name );
        _dbgLogMsg( DBG_READ_DEFINITION, "  Reading array %s[%d] = %ld", 
                    name, cnt, (*value)[cnt] );
        cnt ++;
    }

    // Return, value read
    return( arLen );
}
