/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*

    File : dbgLog.c

    Description : (see include file)

    Created : 9/15/98
    By      : Patrick McDaniel

    Change History
    ----
    09/15/98 PDM File Created
    10/18/98 Completed Windowed Revocation implementation : DPM

*/

/* System Include Files */
#include <stdio.h>
#include <fstream.h>
#include <time.h>
#include <stdarg.h>

/* Project Include Files */
#include <dbgLog.h>

/* Global Data Definitions */
unsigned long    _dbgLogLevel = DEFAULT_DBG_LEVEL;
unsigned short   echoTerm = true;
unsigned short   echoLog = true;
unsigned char   *logFilename = DEFAULT_LOG_FILENAME;

/* Static Data Definitions */
static ofstream *logFile = NULL;
unsigned short   logCount = 0;

/* Create a structure containing text levels */
struct levelText {
  const char *text;
  unsigned long level;
};
struct levelText lText[32] = 
{

    { "DBG_INFORM",             0x00000001 },
    { "DBG_FATAL_ERROR",        0x00000002 },
    { "DBG_MAJOR_ERROR",        0x00000004 },
    { "DBG_MINOR_ERROR",        0x00000008 },
         
    { "DBG_VERIFICATION",       0x00000010 },
    { "DBG_MONITOR",            0x00000020 },
    { "",                       0x00000040 },
    { "",                       0x00000080 },
         
    { "",                       0x00000100 },
    { "",                       0x00000200 },
    { "",                       0x00000400 },
    { "",                       0x00000800 },
         
    { "",                       0x00001000 },
    { "",                       0x00002000 },
    { "",                       0x00004000 },
    { "",                       0x00008000 },
         
    { "DBG_TRAFFIC_GEN",        0x00010000 },
    { "DBG_SIMULATE",           0x00020000 },
    { "DBG_READ_DEFINITION",    0x00040000 },
    { "DBG_TRACE_CACHE",        0x00080000 },
      
    { "DBG_REQUEST_PROCESSING", 0x00100000 },
    { "DBG_CRL_CACHE",          0x00200000 },
    { "DBG_CRL_PROCESSING",     0x00400000 },
    { "",                       0x00800000 },
         
    { "",                       0x01000000 },
    { "",                       0x02000000 },
    { "",                       0x04000000 },
    { "",                       0x08000000 },
         
    { "",                       0x10000000 },
    { "",                       0x20000000 },
    { "",                       0x40000000 },
    { "",                       0x80000000 }

};


/*

   Function     : _dbgLog
   Description  : Write a message to the log.

   Inputs       : dbgLevel - log level of message
                  logEntry - message to write to log
   Outputs      : 0 if successful, -1 if failure

*/

int _dbgLog( int dbgLevel, char *logEntry )
{
    /* If not turned on, ignore */
    if ( !LOG_LEVEL_ON(dbgLevel) )
        return( 0 );

    /* Check the file object is exists */
    if ( logFile == NULL )
       logFile = new ofstream( (const char *)logFilename );

    /* If logFile bad, return unsuccessfull */
    if ( ! logFile )
       return( -1 );

    /* Get Date string, write to log */
    time_t timeVal = time(NULL);
    char *timeStr = ctime(&timeVal);
    timeStr[strlen(timeStr)-1] = 0x0;  /* Trim CR/LF character */
    if ( echoLog )
        *logFile << timeStr << " : " << logEntry << endl;

    /* If needed, echo to terminal */
    if ( echoTerm )
      cout << timeStr << " : " << logEntry << endl;

    /* Check if we need to flush */
    if ( logCount >= DBG_LOG_MSG_FLUSH )
    {
        /* Flush the log to disk */
        logFile->flush();
        logCount = 0;
    }

    /* Return successfully */
    return( 0 );
}


/*

   Function     : _dbgLogMsg
   Description  : Write a message to the log (with formatting).

   Inputs       : lLevel - log level of message
                  logEntry - message to write to log
   Outputs      : 0 if successful, -1 if failure

*/

int _dbgLogMsg( int lLevel, char *frmt, ... )
{
    /* Ignore if not going to log */
    if ( ! LOG_LEVEL_ON(lLevel) )
        return( 0 );

    /* Create var args string and call other log message routine */
    static char tempstr[1024];
    va_list ap;
    va_start( ap, frmt );
#if defined(NO_VSNPRINTF)
    vsprintf( tempstr, frmt, ap );
#else /*NO_VSNPRINTF*/
    vsnprintf( tempstr, 1025, frmt, ap );
#endif /*NO_VSNPRINTF*/
    va_end( ap );
    return( _dbgLog(lLevel, (char *)tempstr) );
}

/*

   Function     : closeDbgLog
   Description  : Close the log file.

   Inputs       : none
   Outputs      : none

*/

void closeDbgLog( void )
{
    /* If already exists */
    if ( logFile != NULL )
    {
        /* Flush, Close, and free it */
        logFile->flush();
        logFile->close();
        delete logFile;
        logFile = NULL;
    }

    /* Return, no return code */
    return;
}

/*

   Function     : fromByteString
   Description  : create string of bytes suitable for printing.

   Inputs       : str - pointer to byte buffer
                  len - number of bytes to include
   Outputs      : a pointer to the constructed string

*/

const char *fromByteString( const char *str, int len )
{
    /* Loop through bytes, printing them to the string */
    static char byteString[1024];
    int index = 0, idx = 0;
    while ( (index < 1020) && (idx < len) )
    {
        sprintf( &byteString[index], "%02x ", (unsigned char)str[idx++] );
        index += 3;
    }
    byteString[index] = 0x0;

    /* Return the string */
    return( byteString );
}

/*

   Function     : textLogLevel
   Description  : convert the string of into a log level 
                  (exit on any unknown debug levels)

   Inputs       : str - pointer to byte buffer
   Outputs      : the log level, or 0 if failure

*/

unsigned long textLogLevel( char *str )
{
    // Convert the string into "|" divided tokens
    unsigned long lLevel = 0;
    char *tok = strtok( str, ":" );
    int found = 0;
    while (tok != NULL)
    {
        // Loop through
        for ( int i = 0; i < 32; i++ )
            if ( strcmp(tok,lText[i].text) == 0 )
            {
                // Found it, set log level
                lLevel |= lText[i].level;
                found = 1;
                break; // out of loop
            }

        // If not found, exit on error
        if ( ! found )
        {
            // Error out
            fprintf( stderr, "UNKNOWN DEBUG LEVEL : [%s], exiting", tok );
            exit( -1 );
        }

        // Get the next token
        found = 0;
        tok = strtok( NULL, ":" );
    }

    // Return the string
    return( lLevel );
}
